import { HttpMethod, Route, StatusCodes } from 'org.crazydoctor.expressts';
import { Request, Response } from 'express';
import { ISession } from '../session/ISession';
import ServerApp from '..';

class PostUpdateComment extends Route {
	private AdminLogin = 'Admin';

	protected action = (req: Request, res: Response): any => {
		const session = req.session as ISession;
		const params = req.body;

		const className = params.class.trim();
		const propertyName = params.property.trim();
		const comment = params.comment;

		if(!session.isAdmin) {
			res.status(StatusCodes.FORBIDDEN).send('Access denied');
			return;
		}

		const db = ServerApp.getDb();

		db.get('SELECT * FROM comments WHERE class = ? AND property = ?', [className, propertyName], (err: any, row: any) => {
			if(err) {
				res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
				return;
			}

			if(row) {
				if(comment.length > 0) {
					db.run('UPDATE comments SET comment = ?, timestamp = ? WHERE class = ? AND property = ?', [comment, new Date().getTime(), className, propertyName], function(err) {
						if(err) {
							res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
							return;
						}
						res.status(StatusCodes.OK).send('UPDATED');
					});
				} else {
					db.run('DELETE FROM comments WHERE class = ? AND property = ?', [className, propertyName], function(err) {
						if(err) {
							res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
							return;
						}
						res.status(StatusCodes.OK).send('DELETED');
					});
				}
			} else {
				db.run('INSERT INTO comments (class, property, comment, timestamp) VALUES (?, ?, ?, ?)', [className, propertyName, comment, new Date().getTime()], function(err) {
					if(err) {
						res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
						return;
					}
					res.status(StatusCodes.CREATED).send('CREATED');
				});
			}
		});
	};

	protected method = HttpMethod.POST;
	protected order = 5;
	protected route = '/updateComment';
}

export default PostUpdateComment;